@@ -1,7 +1,7 @@
 /*
  * drill.c
  * the main file of drill
- * (c) 2005,2006 NLnet Labs
+ * (c) 2005-2008 NLnet Labs
  *
  * See the file LICENSE for the license
  *
@@ -76,7 +76,7 @@ version(FILE *stream, const char *progname)
 {
         fprintf(stream, "%s version %s (ldns version %s)\n", progname, DRILL_VERSION, ldns_version());
         fprintf(stream, "Written by NLnet Labs.\n");
-        fprintf(stream, "\nCopyright (c) 2004-2006 NLnet Labs.\n");
+        fprintf(stream, "\nCopyright (c) 2004-2008 NLnet Labs.\n");
         fprintf(stream, "Licensed under the revised BSD license.\n");
         fprintf(stream, "There is NO warranty; not even for MERCHANTABILITY or FITNESS\n");
         fprintf(stream, "FOR A PARTICULAR PURPOSE.\n");
@@ -144,16 +144,31 @@ main(int argc, char *argv[])
 	ldns_rdf *trace_start_name = NULL;
 
 	int		result = 0;
-	
+
+#ifdef USE_WINSOCK
+	int r;
+	WSADATA wsa_data;
+#endif
+
 	int_type = -1; serv = NULL; type = 0; 
 	int_clas = -1; name = NULL; clas = 0;
 	qname = NULL; 
 	progname = strdup(argv[0]);
+
+#ifdef USE_WINSOCK
+	r = WSAStartup(MAKEWORD(2,2), &wsa_data);
+	if(r != 0) {
+		printf("Failed WSAStartup: %d\n", r);
+		result = EXIT_FAILURE;
+		goto exit;
+	}
+#endif /* USE_WINSOCK */
+		
 	
 	PURPOSE = DRILL_QUERY;
 	qflags = LDNS_RD;
 	qport = LDNS_PORT;
-	verbosity = 0;
+	verbosity = 2;
 	qdnssec = false;
 	qfamily = LDNS_RESOLV_INETANY;
 	qfail = false;
@@ -559,7 +574,7 @@ main(int argc, char *argv[])
 			}
 			qname = ldns_dname_new_frm_str(name);
 			if (!qname) {
-				error("%s", "making qname");
+				error("%s", "parsing query name");
 			}
 			/* don't care about return packet */
 			(void)do_trace(res, qname, type, clas);
@@ -594,7 +609,7 @@ main(int argc, char *argv[])
 				error("%s", "error pkt sending");
 				result = EXIT_FAILURE;
 			} else {
-				if (verbosity != -1) {
+				if (verbosity >= 3) {
 					ldns_pkt_print(stdout, pkt);
 				}
 				
@@ -604,7 +619,8 @@ main(int argc, char *argv[])
 					ldns_resolver_set_dnssec_anchors(res, ldns_rr_list_clone(key_list));
 					result = do_chase(res, qname, type,
 					                  clas, key_list, 
-					                  pkt, qflags, NULL);
+					                  pkt, qflags, NULL,
+								   verbosity);
 					if (result == LDNS_STATUS_OK) {
 						if (verbosity != -1) {
 							mesg("Chase successful");
@@ -883,6 +899,9 @@ main(int argc, char *argv[])
 	ERR_free_strings();
 	EVP_cleanup();
 #endif
+#ifdef USE_WINSOCK
+	WSACleanup();
+#endif
 
 	return result;
 }
