@@ -16,9 +16,15 @@
 
 #include <limits.h>
 
+#ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
+#endif
+#ifdef HAVE_ARPA_INET_H
 #include <arpa/inet.h>
+#endif
+#ifdef HAVE_NETDB_H
 #include <netdb.h>
+#endif
 #include <time.h>
 #include <sys/time.h>
 
@@ -459,7 +465,9 @@ ldns_rdf2buffer_str_wks(ldns_buffer *output, const ldns_rdf *rdf)
 		ldns_buffer_printf(output, "%u ", protocol_nr);
 	}
 
+#ifdef HAVE_ENDPROTOENT
 	endprotoent();
+#endif
 	
 	for (current_service = 0; 
 	     current_service < ldns_rdf_size(rdf) * 7; current_service++) {
@@ -471,7 +479,9 @@ ldns_rdf2buffer_str_wks(ldns_buffer *output, const ldns_rdf *rdf)
 			} else {
 				ldns_buffer_printf(output, "%u ", current_service);
 			}
+#ifdef HAVE_ENDSERVENT
 			endservent();
+#endif
 		}
 	}
 	return ldns_buffer_status(output);
@@ -1145,9 +1155,7 @@ ldns_key2buffer_str(ldns_buffer *output, const ldns_key *k)
 			case LDNS_SIGN_RSASHA1:
 			case LDNS_SIGN_RSASHA1_NSEC3:
 			case LDNS_SIGN_RSASHA256:
-			case LDNS_SIGN_RSASHA256_NSEC3:
 			case LDNS_SIGN_RSASHA512:
-			case LDNS_SIGN_RSASHA512_NSEC3:
 			case LDNS_SIGN_RSAMD5:
 				/* copied by looking at dnssec-keygen output */
 				/* header */
@@ -1156,21 +1164,33 @@ ldns_key2buffer_str(ldns_buffer *output, const ldns_key *k)
 				ldns_buffer_printf(output,"Private-key-format: v1.2\n");
 				switch(ldns_key_algorithm(k)) {
 				case LDNS_SIGN_RSAMD5:
-					ldns_buffer_printf(output,"Algorithm: 1 (RSA)\n");
+					ldns_buffer_printf(output,
+								    "Algorithm: %u (RSA)\n",
+								    LDNS_RSAMD5);
 					break;
 				case LDNS_SIGN_RSASHA1:
 				case LDNS_SIGN_RSASHA1_NSEC3:
-					ldns_buffer_printf(output,"Algorithm: 5 (RSASHA1)\n");
+					ldns_buffer_printf(output,
+								    "Algorithm: %u (RSASHA1)\n",
+								    LDNS_RSASHA1);
 					break;
+#ifdef USE_SHA2
 				case LDNS_SIGN_RSASHA256:
-				case LDNS_SIGN_RSASHA256_NSEC3:
-					ldns_buffer_printf(output,"Algorithm: 6 (RSASHA256)\n");
+					ldns_buffer_printf(output,
+								    "Algorithm: %u (RSASHA256)\n",
+								    LDNS_RSASHA256);
 					break;
 				case LDNS_SIGN_RSASHA512:
-				case LDNS_SIGN_RSASHA512_NSEC3:
-					ldns_buffer_printf(output,"Algorithm: 7 (RSASHA512)\n");
+					ldns_buffer_printf(output,
+								    "Algorithm: %u (RSASHA512)\n",
+								    LDNS_RSASHA512);
 					break;
+#endif
 				default:
+					fprintf(stderr, "Warning: unknown signature ");
+					fprintf(stderr,
+						   "algorithm type %u\n", 
+						   ldns_key_algorithm(k));
 					break;
 				}
 				
