@@ -7,12 +7,13 @@
 
 #include <ldns/dnssec.h>
 
-typedef struct ldns_dnssec_data_chain_struct ldns_dnssec_data_chain;
 /**
  * Chain structure that contains all DNSSEC data needed to
  * verify an rrset
  */
-struct ldns_dnssec_data_chain_struct {
+typedef struct ldns_dnssec_data_chain_struct ldns_dnssec_data_chain;
+struct ldns_dnssec_data_chain_struct
+{
 	ldns_rr_list *rrset;
 	ldns_rr_list *signatures;
 	ldns_rr_type parent_type;
@@ -36,7 +37,8 @@ ldns_dnssec_data_chain *ldns_dnssec_data_chain_new();
 void ldns_dnssec_data_chain_free(ldns_dnssec_data_chain *chain);
 
 /**
- * Frees a dnssec_data_chain structure, and all data contained therein
+ * Frees a dnssec_data_chain structure, and all data
+ * contained therein
  *
  * \param[in] *chain The dnssec_data_chain to free
  */
@@ -51,9 +53,19 @@ void ldns_dnssec_data_chain_deep_free(ldns_dnssec_data_chain *chain);
 void ldns_dnssec_data_chain_print(FILE *out, const ldns_dnssec_data_chain *chain);
 
 /**
- * the data set will be cloned
- * the pkt is optional, can contain the original packet 
+ * Build an ldns_dnssec_data_chain, which contains all
+ * DNSSEC data that is needed to derive the trust tree later
+ *
+ * The data_set will be cloned
+ *
+ * \param[in] *res resolver structure for further needed queries
+ * \param[in] qflags resolution flags
+ * \param[in] *data_set The original rrset where the chain ends
+ * \param[in] *pkt optional, can contain the original packet
  * (and hence the sigs and maybe the key)
+ * \param[in] *orig_rr The original Resource Record
+ *
+ * \return the DNSSEC data chain
  */
 ldns_dnssec_data_chain *ldns_dnssec_build_data_chain(ldns_resolver *res,
 										   const uint16_t qflags,
@@ -62,13 +74,15 @@ ldns_dnssec_data_chain *ldns_dnssec_build_data_chain(ldns_resolver *res,
 										   ldns_rr *orig_rr);
 
 /**
- * Tree structure that contains the relation of DNSSEC data, and their cryptographic
- * status.
+ * Tree structure that contains the relation of DNSSEC data,
+ * and their cryptographic status.
  *
- * This tree is derived from a data_chain, and can be used to look whether there is a
- * connection between an RRSET and a trusted key. The tree only contains pointers to
- * the data_chain, and therefore one should *never* free() the data_chain when there is
- * still a trust tree derived from that chain.
+ * This tree is derived from a data_chain, and can be used
+ * to look whether there is a connection between an RRSET
+ * and a trusted key. The tree only contains pointers to the
+ * data_chain, and therefore one should *never* free() the
+ * data_chain when there is still a trust tree derived from
+ * that chain.
  *
  * Example tree:
  *     key   key    key
@@ -83,19 +97,21 @@ ldns_dnssec_data_chain *ldns_dnssec_build_data_chain(ldns_resolver *res,
  *            |
  *            rr
  *
- * For each signature there is a parent; if the parent pointer is null, it
- * couldn't be found and there was no denial; otherwise is a tree which
- * contains either a DNSKEY, a DS, or a NSEC rr;
+ * For each signature there is a parent; if the parent
+ * pointer is null, it couldn't be found and there was no
+ * denial; otherwise is a tree which contains either a
+ * DNSKEY, a DS, or a NSEC rr
  */
 typedef struct ldns_dnssec_trust_tree_struct ldns_dnssec_trust_tree;
-struct ldns_dnssec_trust_tree_struct {
+struct ldns_dnssec_trust_tree_struct
+{
 	ldns_rr *rr;
 	/* the complete rrset this rr was in */
 	ldns_rr_list *rrset;
 	ldns_dnssec_trust_tree *parents[LDNS_DNSSEC_TRUST_TREE_MAX_PARENTS];
 	ldns_status parent_status[LDNS_DNSSEC_TRUST_TREE_MAX_PARENTS];
-	/** for debugging, add signatures too (you might want those if they 
-	    contain errors) */
+	/** for debugging, add signatures too (you might want
+	    those if they contain errors) */
 	ldns_rr *parent_signature[LDNS_DNSSEC_TRUST_TREE_MAX_PARENTS];
 	size_t parent_count;
 };
@@ -109,8 +125,9 @@ ldns_dnssec_trust_tree *ldns_dnssec_trust_tree_new();
 
 /**
  * Frees the dnssec_trust_tree recursively
- * There is no deep free; all data in the trust tree consists of pointers
- * to a data_chain
+ *
+ * There is no deep free; all data in the trust tree
+ * consists of pointers to a data_chain
  *
  * \param[in] tree The tree to free
  */
@@ -125,10 +142,11 @@ void ldns_dnssec_trust_tree_free(ldns_dnssec_trust_tree *tree);
 size_t ldns_dnssec_trust_tree_depth(ldns_dnssec_trust_tree *tree);
 
 /**
- * Prints the dnssec_trust_tree structure to the given file stream
- * Each line is prepended by 2*tabs spaces
- * If a link status is not LDNS_STATUS_OK; the status and relevant
- * signatures are printed too
+ * Prints the dnssec_trust_tree structure to the given file
+ * stream.
+ *
+ * If a link status is not LDNS_STATUS_OK; the status and
+ * relevant signatures are printed too
  *
  * \param[in] *out The file stream to print to
  * \param[in] tree The trust tree to print
@@ -150,33 +168,38 @@ void ldns_dnssec_trust_tree_print(FILE *out,
  * \param[in] parent_status The DNSSEC status for this parent, child and RRSIG
  * \return LDNS_STATUS_OK if the addition succeeds, error otherwise
  */
-ldns_status
-ldns_dnssec_trust_tree_add_parent(ldns_dnssec_trust_tree *tree,
-                                  const ldns_dnssec_trust_tree *parent,
-                                  const ldns_rr *parent_signature,
-                                  const ldns_status parent_status);
+ldns_status ldns_dnssec_trust_tree_add_parent(ldns_dnssec_trust_tree *tree,
+									 const ldns_dnssec_trust_tree *parent,
+									 const ldns_rr *parent_signature,
+									 const ldns_status parent_status);
 
 /**
- * Generates a dnssec_trust_ttree for the given rr from the given data_chain
- * Don't free the data_chain before you are done with this tree
+ * Generates a dnssec_trust_ttree for the given rr from the
+ * given data_chain
+ *
+ * This does not clone the actual data; Don't free the
+ * data_chain before you are done with this tree
  *
  * \param[in] *data_chain The chain to derive the trust tree from
  * \param[in] *rr The RR this tree will be about
  * \return ldns_dnssec_trust_tree *
  */
-ldns_dnssec_trust_tree *ldns_dnssec_derive_trust_tree(ldns_dnssec_data_chain *data_chain, ldns_rr *rr);
+ldns_dnssec_trust_tree *ldns_dnssec_derive_trust_tree(
+                            ldns_dnssec_data_chain *data_chain,
+					   ldns_rr *rr);
 
 /**
- * Sub function for derive_trust_tree that is used for a 'normal' rrset
+ * Sub function for derive_trust_tree that is used for a
+ * 'normal' rrset
  *
  * \param[in] new_tree The trust tree that we are building
  * \param[in] data_chain The data chain containing the data for the trust tree
  * \param[in] cur_sig_rr The currently relevant signature
  */
-void
-ldns_dnssec_derive_trust_tree_normal_rrset(ldns_dnssec_trust_tree *new_tree,
-                                           ldns_dnssec_data_chain *data_chain,
-                                           ldns_rr *cur_sig_rr);
+void ldns_dnssec_derive_trust_tree_normal_rrset(
+         ldns_dnssec_trust_tree *new_tree,
+	    ldns_dnssec_data_chain *data_chain,
+	    ldns_rr *cur_sig_rr);
 
 /**
  * Sub function for derive_trust_tree that is used for DNSKEY rrsets
@@ -186,11 +209,11 @@ ldns_dnssec_derive_trust_tree_normal_rrset(ldns_dnssec_trust_tree *new_tree,
  * \param[in] cur_rr The currently relevant DNSKEY RR
  * \param[in] cur_sig_rr The currently relevant signature
  */
-void
-ldns_dnssec_derive_trust_tree_dnskey_rrset(ldns_dnssec_trust_tree *new_tree,
-                                           ldns_dnssec_data_chain *data_chain,
-                                           ldns_rr *cur_rr,
-                                           ldns_rr *cur_sig_rr);
+void ldns_dnssec_derive_trust_tree_dnskey_rrset(
+         ldns_dnssec_trust_tree *new_tree,
+	    ldns_dnssec_data_chain *data_chain,
+	    ldns_rr *cur_rr,
+	    ldns_rr *cur_sig_rr);
 
 /**
  * Sub function for derive_trust_tree that is used for DS rrsets
@@ -199,10 +222,10 @@ ldns_dnssec_derive_trust_tree_dnskey_rrset(ldns_dnssec_trust_tree *new_tree,
  * \param[in] data_chain The data chain containing the data for the trust tree
  * \param[in] cur_rr The currently relevant DS RR
  */
-void
-ldns_dnssec_derive_trust_tree_ds_rrset(ldns_dnssec_trust_tree *new_tree,
-                                       ldns_dnssec_data_chain *data_chain,
-                                       ldns_rr *cur_rr);
+void ldns_dnssec_derive_trust_tree_ds_rrset(
+         ldns_dnssec_trust_tree *new_tree,
+	    ldns_dnssec_data_chain *data_chain,
+	    ldns_rr *cur_rr);
 
 /**
  * Sub function for derive_trust_tree that is used when there are no
@@ -211,9 +234,9 @@ ldns_dnssec_derive_trust_tree_ds_rrset(ldns_dnssec_trust_tree *new_tree,
  * \param[in] new_tree The trust tree that we are building
  * \param[in] data_chain The data chain containing the data for the trust tree
  */
-void
-ldns_dnssec_derive_trust_tree_no_sig(ldns_dnssec_trust_tree *new_tree,
-                                     ldns_dnssec_data_chain *data_chain);
+void ldns_dnssec_derive_trust_tree_no_sig(
+         ldns_dnssec_trust_tree *new_tree,
+	    ldns_dnssec_data_chain *data_chain);
 
 /**
  * Returns OK if there is a trusted path in the tree to one of 
@@ -225,8 +248,9 @@ ldns_dnssec_derive_trust_tree_no_sig(ldns_dnssec_trust_tree *new_tree,
  *                        the keys, or the *first* error encountered
  *                        if there were no paths
  */
-ldns_status ldns_dnssec_trust_tree_contains_keys(ldns_dnssec_trust_tree *tree,
-									    ldns_rr_list *keys);
+ldns_status ldns_dnssec_trust_tree_contains_keys(
+			 ldns_dnssec_trust_tree *tree,
+			 ldns_rr_list *keys);
 
 /**
  * Verifies a list of signatures for one rrset.
@@ -258,11 +282,10 @@ ldns_status ldns_verify(ldns_rr_list *rrset,
  * \return the set of trusted keys for the domain, or NULL if no 
  *         trust path could be built.
  */
-ldns_rr_list *
-ldns_fetch_valid_domain_keys(const ldns_resolver * res,
-					    const ldns_rdf * domain,
-					    const ldns_rr_list * keys,
-					    ldns_status *status);
+ldns_rr_list *ldns_fetch_valid_domain_keys(const ldns_resolver * res,
+								   const ldns_rdf * domain,
+								   const ldns_rr_list * keys,
+								   ldns_status *status);
 
 /**
  * Validates the DNSKEY RRset for the given domain using the provided 
@@ -274,10 +297,9 @@ ldns_fetch_valid_domain_keys(const ldns_resolver * res,
  * \return the set of trusted keys for the domain, or NULL if the RRSET
  *         could not be validated
  */
-ldns_rr_list *
-ldns_validate_domain_dnskey (const ldns_resolver *res,
-					    const ldns_rdf *domain,
-					    const ldns_rr_list *keys);
+ldns_rr_list *ldns_validate_domain_dnskey (const ldns_resolver *res,
+								   const ldns_rdf *domain,
+								   const ldns_rr_list *keys);
 
 /**
  * Validates the DS RRset for the given domain using the provided trusted keys.
@@ -287,11 +309,10 @@ ldns_validate_domain_dnskey (const ldns_resolver *res,
  * \param[in] keys the current set of trusted keys
  * \return the set of trusted keys for the domain, or NULL if the RRSET could not be validated
  */
-ldns_rr_list *
-ldns_validate_domain_ds(const ldns_resolver *res,
-				    const ldns_rdf *
-				    domain,
-				    const ldns_rr_list * keys);
+ldns_rr_list *ldns_validate_domain_ds(const ldns_resolver *res,
+							   const ldns_rdf *
+							   domain,
+							   const ldns_rr_list * keys);
 
 /**
  * Verifies a list of signatures for one RRset using a valid trust path.
@@ -304,11 +325,10 @@ ldns_validate_domain_ds(const ldns_resolver *res,
  *                              the signatures are added to it
  * \return status LDNS_STATUS_OK if there is at least one correct key
  */
-ldns_status
-ldns_verify_trusted(ldns_resolver *res,
-				ldns_rr_list *rrset,
-				ldns_rr_list *rrsigs,
-				ldns_rr_list *validating_keys);
+ldns_status ldns_verify_trusted(ldns_resolver *res,
+						  ldns_rr_list *rrset,
+						  ldns_rr_list *rrsigs,
+						  ldns_rr_list *validating_keys);
 
 /**
  * denial is not just a river in egypt
@@ -320,10 +340,9 @@ ldns_verify_trusted(ldns_resolver *res,
  * \return LDNS_STATUS_OK if the NSEC RRs deny the existence, error code
  *                        containing the reason they do not otherwise
  */
-ldns_status
-ldns_dnssec_verify_denial(ldns_rr *rr,
-                          ldns_rr_list *nsecs,
-                          ldns_rr_list *rrsigs);
+ldns_status ldns_dnssec_verify_denial(ldns_rr *rr,
+							   ldns_rr_list *nsecs,
+							   ldns_rr_list *rrsigs);
 
 /**
  * Denial of existence using NSEC3 records
@@ -342,13 +361,12 @@ ldns_dnssec_verify_denial(ldns_rr *rr,
  * \return LDNS_STATUS_OK if the NSEC3 RRs deny the existence, error code
  *                        containing the reason they do not otherwise
  */
-ldns_status
-ldns_dnssec_verify_denial_nsec3(ldns_rr *rr,
-						  ldns_rr_list *nsecs,
-						  ldns_rr_list *rrsigs,
-						  ldns_pkt_rcode packet_rcode,
-						  ldns_rr_type packet_qtype,
-						  bool packet_nodata);
+ldns_status ldns_dnssec_verify_denial_nsec3(ldns_rr *rr,
+								    ldns_rr_list *nsecs,
+								    ldns_rr_list *rrsigs,
+								    ldns_pkt_rcode packet_rcode,
+								    ldns_rr_type packet_qtype,
+								    bool packet_nodata);
 
 /**
  * Verifies the already processed data in the buffers
@@ -394,7 +412,10 @@ ldns_status ldns_verify_rrsig_buffers_raw(unsigned char* sig,
  * \return a list of keys which validate the rrsig + rrset. Returns NULL
  *         when none of the keys validate.
  */
-ldns_status ldns_verify_rrsig_keylist(ldns_rr_list *rrset, ldns_rr *rrsig, const ldns_rr_list *keys, ldns_rr_list *good_keys);
+ldns_status ldns_verify_rrsig_keylist(ldns_rr_list *rrset,
+							   ldns_rr *rrsig,
+							   const ldns_rr_list *keys,
+							   ldns_rr_list *good_keys);
 
 /**
  * convert dsa data
@@ -414,7 +435,9 @@ ldns_convert_dsa_rrsig_rdata(ldns_buffer *target_buffer,
  * \param[in] key the key to use
  * \return status message wether verification succeeded.
  */
-ldns_status ldns_verify_rrsig(ldns_rr_list *rrset, ldns_rr *rrsig, ldns_rr *key);
+ldns_status ldns_verify_rrsig(ldns_rr_list *rrset,
+						ldns_rr *rrsig,
+						ldns_rr *key);
 
 /**
  * verifies a buffer with signature data for a buffer with rrset data 
