@@ -15,10 +15,18 @@
 
 #include <ldns/ldns.h>
 
+#ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
+#endif
+#ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
+#endif
+#ifdef HAVE_NETDB_H
 #include <netdb.h>
+#endif
+#ifdef HAVE_ARPA_INET_H
 #include <arpa/inet.h>
+#endif
 #include <sys/time.h>
 #include <errno.h>
 
@@ -41,14 +49,6 @@ ldns_send(ldns_pkt **result_packet, ldns_resolver *r, const ldns_pkt *query_pkt)
 		result = LDNS_STATUS_ERR;
 	} else {
         	result = ldns_send_buffer(result_packet, r, qb, tsig_mac);
-/*
-printf(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>\n");
-ldns_pkt_print(stdout, query_pkt);
-printf(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>\n\n");
-printf("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<\n");
-ldns_pkt_print(stdout, *result_packet);
-printf("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<\n\n");
-*/
 	}
 
 	ldns_buffer_free(qb);
@@ -64,7 +64,7 @@ ldns_send_buffer(ldns_pkt **result, ldns_resolver *r, ldns_buffer *qb, ldns_rdf
 	struct sockaddr_storage *ns;
 	size_t ns_len;
 	struct timeval tv_s;
-        struct timeval tv_e;
+	struct timeval tv_e;
 
 	ldns_rdf **ns_array;
 	size_t *rtt;
@@ -92,7 +92,6 @@ ldns_send_buffer(ldns_pkt **result, ldns_resolver *r, ldns_buffer *qb, ldns_rdf
 
 	/* loop through all defined nameservers */
 	for (i = 0; i < ldns_resolver_nameserver_count(r); i++) {
-
 		if (rtt[i] == LDNS_RESOLV_RTT_INF) {
 			/* not reachable nameserver! */
 			continue;
@@ -120,6 +119,7 @@ ldns_send_buffer(ldns_pkt **result, ldns_resolver *r, ldns_buffer *qb, ldns_rdf
 		gettimeofday(&tv_s, NULL);
 
 		send_status = LDNS_STATUS_ERR;
+
 		/* reply_bytes implicitly handles our error */
 		if (1 == ldns_resolver_usevc(r)) {
 			for (retries = ldns_resolver_retry(r); retries > 0; retries--) {
@@ -207,7 +207,7 @@ ldns_send_buffer(ldns_pkt **result, ldns_resolver *r, ldns_buffer *qb, ldns_rdf
 		}
 	}
 #else
-	tsig_mac = tsig_mac;
+	(void)tsig_mac;
 #endif /* HAVE_SSL */
 	
 	LDNS_FREE(reply_bytes);
@@ -272,7 +272,7 @@ ldns_udp_connect(const struct sockaddr_storage *to, struct timeval timeout)
 			== -1) {
                 return 0;
         }
-	if (setsockopt(sockfd, SOL_SOCKET, SO_RCVTIMEO, &timeout,
+	if (setsockopt(sockfd, SOL_SOCKET, SO_RCVTIMEO, (void*)&timeout,
 				(socklen_t)sizeof(timeout))) {
                 /* might fail, in that case, use default for now */
 		/*
@@ -291,22 +291,25 @@ ldns_tcp_connect(const struct sockaddr_storage *to, socklen_t tolen,
 	
 	if ((sockfd = socket((int)((struct sockaddr*)to)->sa_family, SOCK_STREAM, 
 					IPPROTO_TCP)) == -1) {
-		/*perror("could not open socket");*/
 		return 0;
 	}
 
-        if (setsockopt(sockfd, SOL_SOCKET, SO_RCVTIMEO, &timeout,
-                        (socklen_t) sizeof(timeout))) {
-                /*perror("setsockopt");*/
-                close(sockfd);
-                return 0;
-        }
+	if (setsockopt(sockfd, SOL_SOCKET, SO_RCVTIMEO, (void*)&timeout,
+				(socklen_t) sizeof(timeout))) {
+		close(sockfd);
+		return 0;
+	}
+	if (setsockopt(sockfd, SOL_SOCKET, SO_SNDTIMEO, (void*)&timeout,
+				(socklen_t) sizeof(timeout))) {
+		close(sockfd);
+		return 0;
+	}
 
 	if (connect(sockfd, (struct sockaddr*)to, tolen) == -1) {
  		close(sockfd);
-		/*perror("could not bind socket");*/
 		return 0;
 	}
+
 	return sockfd;
 }
 
