@@ -3,7 +3,7 @@
  *
  * Taken from Unbound, modified for ldns
  *
- * Copyright (c) 2001-2007, NLnet Labs. All rights reserved.
+ * Copyright (c) 2001-2008, NLnet Labs. All rights reserved.
  * 
  * This software is open source.
  * 
@@ -100,6 +100,12 @@ ldns_rbtree_init(ldns_rbtree_t *rbtree, int (*cmpf)(const void *, const void *))
 	rbtree->cmp = cmpf;
 }
 
+void 
+ldns_rbtree_free(ldns_rbtree_t *rbtree)
+{
+	free(rbtree);
+}
+
 /*
  * Rotates the node to the left.
  *
