@@ -51,7 +51,7 @@ static int verb = 0;
 static void
 usage()
 {
-	printf("Usage:	unbound-host [-vdh] [-c class] [-t type] hostname\n");
+	printf("Usage:	unbound-host [-vdhr] [-c class] [-t type] hostname\n");
 	printf("                     [-y key] [-f keyfile] [-F namedkeyfile]\n");
 	printf("                     [-C configfile]\n");
 	printf("  Queries the DNS for information.\n");
@@ -65,6 +65,8 @@ usage()
 	printf("    -f keyfile		read trust anchors from file, with lines as -y.\n");
 	printf("    -F keyfile		read named.conf-style trust anchors.\n");
 	printf("    -C config		use the specified unbound.conf\n");
+	printf("    -r			read forwarder information from /etc/resolv.conf\n");
+	printf("      			breaks validation if the fwder does not do DNSSEC.\n");
 	printf("    -v			be more verbose, shows nodata and security.\n");
 	printf("    -d			debug, traces the action, -d -d shows more.\n");
 	printf("    -h			show this usage help.\n");
@@ -411,7 +413,7 @@ int main(int argc, char* argv[])
 	}
 
 	/* parse the options */
-	while( (c=getopt(argc, argv, "F:c:df:ht:vy:C:")) != -1) {
+	while( (c=getopt(argc, argv, "F:c:df:hrt:vy:C:")) != -1) {
 		switch(c) {
 		case 'c':
 			qclass = optarg;
@@ -425,6 +427,9 @@ int main(int argc, char* argv[])
 				debuglevel = 2; /* at least VERB_DETAIL */
 			check_ub_res(ub_ctx_debuglevel(ctx, debuglevel));
 			break;
+		case 'r':
+			check_ub_res(ub_ctx_resolvconf(ctx, "/etc/resolv.conf"));
+			break;
 		case 't':
 			qtype = optarg;
 			break;
