@@ -14,12 +14,18 @@
 
 #include <ldns/ldns.h>
 
+#ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
+#endif
+#ifdef HAVE_ARPA_INET_H
 #include <arpa/inet.h>
+#endif
 #include <time.h>
 
 #include <errno.h>
+#ifdef HAVE_NETDB_H
 #include <netdb.h> 
+#endif
 
 #include <limits.h>
 #ifdef HAVE_SYS_PARAM_H
@@ -230,7 +236,7 @@ ldns_str2rdf_dname(ldns_rdf **d, const char *str)
 	
 	len = strlen((char*)str);
 	/* octet representation can make strings a lot longer than actual length */
-	if (len > LDNS_MAX_DOMAINLEN * 3) {
+	if (len > LDNS_MAX_DOMAINLEN * 4) {
 		return LDNS_STATUS_DOMAINNAME_OVERFLOW;
 	}
 	if (0 == len) {
@@ -254,6 +260,9 @@ ldns_str2rdf_dname(ldns_rdf **d, const char *str)
 	pq = buf;
 	label_len = 0;
 	for (s = p = (uint8_t *) str; *s; s++, q++) {
+		if (q > buf + LDNS_MAX_DOMAINLEN) {
+			return LDNS_STATUS_DOMAINNAME_OVERFLOW;
+		}	
 		*q = 0;
 		switch (*s) {
 		case '.':
@@ -995,8 +1004,12 @@ ldns_str2rdf_wks(ldns_rdf **rd, const char *str)
 	LDNS_FREE(bitmap);
 	LDNS_FREE(data);
 	free(proto_str);
+#ifdef HAVE_ENDSERVENT
 	endservent();
+#endif
+#ifdef HAVE_ENDPROTOENT
 	endprotoent();
+#endif
 	
 	return LDNS_STATUS_OK;
 }
