@@ -1106,15 +1106,19 @@ processInit(struct module_qstate* qstate, struct val_qstate* vq,
 			rrsets[vq->rrset_skip]->rk.type);
 		vq->qchase.qclass = ntohs(vq->orig_msg->rep->
 			rrsets[vq->rrset_skip]->rk.rrset_class);
-		/* for type DS look at the parent side for keys/trustanchor */
-		/* also for NSEC not at apex */
-		if(vq->qchase.qtype == LDNS_RR_TYPE_DS ||
-			(vq->qchase.qtype == LDNS_RR_TYPE_NSEC && 
-			 !(vq->orig_msg->rep->rrsets[vq->rrset_skip]->
-			 rk.flags&PACKED_RRSET_NSEC_AT_APEX))) {
-			dname_remove_label(&vq->qchase.qname, 
-				&vq->qchase.qname_len);
-		}
+	}
+	lookup_name = vq->qchase.qname;
+	lookup_len = vq->qchase.qname_len;
+	/* for type DS look at the parent side for keys/trustanchor */
+	/* also for NSEC not at apex */
+	if(vq->qchase.qtype == LDNS_RR_TYPE_DS ||
+		(vq->qchase.qtype == LDNS_RR_TYPE_NSEC && 
+		 vq->orig_msg->rep->rrset_count > vq->rrset_skip &&
+		 ntohs(vq->orig_msg->rep->rrsets[vq->rrset_skip]->rk.type) ==
+		 LDNS_RR_TYPE_NSEC &&
+		 !(vq->orig_msg->rep->rrsets[vq->rrset_skip]->
+		 rk.flags&PACKED_RRSET_NSEC_AT_APEX))) {
+		dname_remove_label(&lookup_name, &lookup_len);
 	}
 
 	val_mark_indeterminate(vq->chase_reply, qstate->env->anchors, 
@@ -1123,7 +1127,7 @@ processInit(struct module_qstate* qstate, struct val_qstate* vq,
 	vq->empty_DS_name = NULL;
 	vq->ds_rrset = 0;
 	vq->trust_anchor = anchors_lookup(qstate->env->anchors, 
-		vq->qchase.qname, vq->qchase.qname_len, vq->qchase.qclass);
+		lookup_name, lookup_len, vq->qchase.qclass);
 	if(vq->trust_anchor == NULL) {
 		/*response isn't under a trust anchor, so we cannot validate.*/
 		vq->chase_reply->security = sec_status_indeterminate;
